#!/usr/bin/env lua

-- $Author: bam $
-- $Date: 2007-05-06 08:00:56 -0400 (Sun, 06 May 2007) $ 
-- $Revision: 34624 $

local http = require("socket.http")

local string, table = string, table
local format, match, gmatch = string.format, string.match, string.gmatch
local insert = table.insert
local sort = function(t, ...) table.sort(t, ...) return t end

local function read_data(filename)
  local datafile, err = io.open(filename, "r")
  assert(datafile, err)
  local data = {}
  for line in datafile:lines() do
    local id, price, name = match(line, "^(%d+)\t(%d*)\t(.+)$")
    if id then data[tonumber(id)] = {price = tonumber(price), name = name} end
  end
  datafile:close()
  return data
end

local function write_data(filename, items)
  local datafile, err = io.open(filename, "w+")
  assert(datafile, err)
  local count = 0
  for id = 1, table.maxn(items) do
    local item = items[id]
    if item then
      local price, name = item.price, item.name
      datafile:write(format("%d\t%s\t%s\n", id, price and tostring(price) or "", name))
      count = count + 1
    end
  end
  datafile:close()
  print(format("Wrote %d items to %s", count, filename))
end

local function get_page(url)
  print(format("Fetching page from %s", url))
  local r, e = http.request(url)
  assert(r, e)
  return r
end

local function item_url(id)
  return format("http://www.wowhead.com/?item=%d", id)
end

local function scan_item_urls(page)
  local items, seen = {}, {}
  for idstr in gmatch(page, "?item=(%d+)") do
    if not seen[idstr] then 
      seen[idstr] = true
      insert(items, tonumber(idstr))
    end
  end
  return sort(items)
end

local function scan_coin(str, coinName)
  return tonumber(match(str, format('money%s%%D-(%%d+)', coinName))) or 0
end

local function scan_item_page(page, id)
  local name = match(page, "<title>%s*(.-)%s+-%s+Items%s+-%s+World%sof%sWarcraft</title>")
  if not name or name == "" then print(format("Skipping item:%d: Could not find item name!", id)) return end
  local money_div = match(page, "<%s*div%s*>%s*Sells%s+for%s*(.-)%s*</%s*div%s*>")
  if not money_div or money_div == "" then return name end
  local g, s, c = scan_coin(money_div, "gold"), scan_coin(money_div, "silver"), scan_coin(money_div, "copper")
  return name, g*10000 + s*100 + c
end

local items = read_data("data.csv")
-- write_data("data.csv", items)
-- if true then return end

local latest_additions_page = get_page("http://www.wowhead.com/?latest=additions")

local new_item_urls = {}
for _, id in ipairs(scan_item_urls(latest_additions_page)) do
  local item = items[id]
  if item then
    print(format("Skipping %q (%d): already known!", item.name, id))
  else
    insert(new_item_urls, id)
  end
end
if #new_item_urls == 0 then print("No new items found") return end
print(format("Found %d new item urls...", #new_item_urls))

for i, id in ipairs(new_item_urls) do
  local page = get_page(item_url(id))
  local name, price = scan_item_page(page, id)
  if name then print(format(" *%6s: Found %q (%d) = %s", tostring(i), name, id, price and tostring(price) or "(no price)")) end
  items[id] = {price = price, name = name}
end

write_data("data.csv", items)





